/*
 *  Name        : kdmclock.cpp
 *  Version     : 0.6
 *  Description : A clock with some more...
 *
 *  Copyright (C) 1998 Luca Montecchiani <m.luca@usa.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "kdmclock.h"
#include "kdmclock.moc"

kdmclock::kdmclock( QWidget *parent, const char *name )
	: QWidget( parent, name )
{
	initMetaObject();

	// start internal timer
	QTimer *internalTimer = new QTimer( this );
	connect( internalTimer, SIGNAL(timeout()), SLOT(Timeout()) );
	// internalTimer->start( 1000 );
	internalTimer->start( 100 );

	// reading rc file
	KConfig	*config = kapp->getConfig();

	onmenu = FALSE;

	config->setGroup("Option");
	date 	= config->readNumEntry("date", FALSE);
	second 	= config->readNumEntry("second", TRUE);
	digital = config->readNumEntry("digital", FALSE);
	beep 	= config->readNumEntry("beep", TRUE);
	speak 	= config->readNumEntry("speak", TRUE);
	tree_d 	= config->readNumEntry("tree_d", TRUE);
	border 	= config->readNumEntry("border", FALSE);
//	ver 	= config->readNumEntry("ver", 0);
//	config->writeEntry("ver", VER);

	config->setGroup( "Font" );
	font.setFamily(config->readEntry( "Family", "Utopia"));
	font.setPointSize(config->readNumEntry( "Point Size", 51));
	font.setWeight(config->readNumEntry( "Weight", 75));
	font.setItalic(config->readNumEntry( "Italic",TRUE ));
	font.setBold(config->readNumEntry( "Bold",TRUE ));

	config->setGroup( "Color" );
	QColor colortmp = lightGray;
	Back = config->readColorEntry( "Background" , &colortmp );
	colortmp = black;
	Fore = config->readColorEntry( "Foreground" , &colortmp );
	colortmp = gray;
	Shad = config->readColorEntry( "Shadow" , &colortmp );

	setFixedSize( 100, 100 );

	setBackgroundColor(Back);

//	if ( ver != VER ) Hints();

	// settings up some things
//	Show_ontop();
//	Show_digital();
	if ( !digital )
		{
		if ( height() < width() )
			resize( height(),height());
		else	resize( width() ,width() );
		}
//	Show_second();
//	Show_date();
//	if ( date )
//		{
//		QDate d = QDate::currentDate();
//		setCaption(d.toString());
//		}
//	else	setCaption("kdmclock");
//	Beep();
//	Speak();
//	Show_tree_d();
//	Show_border();
	if ( border )
		recreate(parentWidget(),0,pos(),TRUE);
	else 	recreate(parentWidget(),WStyle_Customize | WState_ClickToFocus, pos(), TRUE);
	//else 	recreate(parentWidget(),WStyle_Customize | WStyle_NoBorder | WState_ClickToFocus, pos(), TRUE);
	Initialize();

	sound = FALSE;
#ifndef KDMCLOCKDEBUG
//	show();
#endif
}


void kdmclock::Timeout()
{
	// get current time
	time = QTime::currentTime();

	// Multimedia Stuff
	if ( time.minute() == 0 )
		{
		if ( !sound )
			{
			sound = TRUE;
			if ( beep )
				kapp->beep();
			if ( speak )
				{
				KAudio KAServer;
				KAServer.setAutosync(TRUE);

				Buf=kapp->kde_datadir()+"/kdmclock/sounds/thetimeis.wav";
				KAServer.play(Buf); 

				Buf.sprintf("/kdmclock/sounds/%02d.wav",time.hour());
				Buf=kapp->kde_datadir()+Buf;
				KAServer.play(Buf); 

				Buf=kapp->kde_datadir()+"/kdmclock/sounds/oclock.wav";
				KAServer.play(Buf); 
				}
			}
		}
	else	sound = FALSE;

	// check if saver is running !!!
//	if ( (ontop) && (!hasFocus()) && (!menu->isVisible()) && (!onmenu) ) raise();

	// force repain for kdmclock area
	repaint();
}

void kdmclock::Initialize()
{
	// Get icon !!
//	QPixmap p1( kdmclock_xpm );
//	setIcon(p1);

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	setBackgroundMode( NoBackground );

	// very sensible !!
//	setFocusPolicy( QWidget::StrongFocus );
}

void kdmclock::paintEvent( QPaintEvent * )
{
	if ( !isVisible() )
		return;

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	QPixmap pm( size() );
	QPainter paint;
	pm.fill( Back );
	paint.begin( &pm );

	if ( !border )
		paint.drawRect(0,0,width(),height());

	if ( digital )
		{
		if ( second )
			Buf.sprintf("%02d:%02d:%02d",time.hour(),time.minute(),time.second());
		else	Buf.sprintf("%02d:%02d",time.hour(),time.minute());
		font.setPointSize(QMIN((int)(width()/Buf.length()*1.5),height()));
		paint.setFont( font );
		if ( tree_d )
			{
			paint.setPen( Shad );
			paint.drawText(3,3,width(),height(),AlignHCenter|AlignVCenter,Buf,-1,0,0);
			}
		paint.setPen( Fore );
		paint.drawText(rect(),AlignHCenter|AlignVCenter,Buf,-1,0,0);
		}
	else 	{
		QPointArray pts;
		QPoint cp = rect().center();
		int d = QMIN(width(),height());
		if ( tree_d )
			{
			paint.setPen( Shad );
			paint.setBrush( Shad );
			paint.setViewport(4,4,width(),height());
			}
		else 	{
			paint.setPen( Fore );
			paint.setBrush( Fore );
			}

		for ( int c=0 ; c < 2 ; c++ )
			{
			QWMatrix matrix;
			matrix.translate( cp.x(), cp.y() );
			matrix.scale( d/1000.0F, d/1000.0F );

			// lancetta delle ore 
			float h_angle = 30*(time.hour()%12-3) + time.minute()/2;
			matrix.rotate( h_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints( 4, -20,0,  0,-20, 300,0, 0,20 );
			paint.drawPolygon( pts );
			matrix.rotate( -h_angle );

			// lancetta dei minuti
			float m_angle = (time.minute()-15)*6;
			matrix.rotate( m_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints( 4, -10,0, 0,-10, 400,0, 0,10 );
			paint.drawPolygon( pts );
			matrix.rotate( -m_angle );

			// lancetta dei secondi
			float s_angle = (time.second()-15)*6;
			matrix.rotate( s_angle );
			paint.setWorldMatrix( matrix );
			pts.setPoints(4,0,0,0,0,400,0,0,0);
			if ( second )
				paint.drawPolygon( pts );
			matrix.rotate( -s_angle );

			// quadrante
			for ( int i=0 ; i < 60 ; i++ )
				{
				paint.setWorldMatrix( matrix );
				if ( (i % 5) == 0 )
					paint.drawLine( 450,0, 500,0 );	// draw hour lines
				else	paint.drawPoint( 480,0 );	// draw second lines
				matrix.rotate( 6 );
				}

			if ( tree_d )
				{
				paint.setPen( Fore );
				paint.setBrush( Fore );
				paint.setViewport(0,0,width(),height());
				}
			else	break;
			}
		}
	paint.end();

	// flicker free code by Remi Guyomarch <rguyom@mail.dotcom.fr>
	bitBlt (this, 0,0, &pm); 
}

// ***********************************************************************
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// >>>>>>>>>>>>>>>>>>>>>>>>> Our trip start here <<<<<<<<<<<<<<<<<<<<<<<<<
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// ***********************************************************************
#ifdef KDMCLOCKDEBUG
int main( int argc, char **argv )
{
	KApplication app( argc, argv );
	kdmclock *clock = new kdmclock;
	app.setMainWidget( clock );
	clock->show();
	app.exec();
}
#endif
